/*
 * Copyright (c) 1993,1994,1995,1997,1998
 *      Texas A&M University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Texas A&M University
 *      and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Developers:
 *     Russell Neeper, David K. Hess, Douglas Lee Schales, David R. Safford
 */

#include <sys/types.h>
#include <stdio.h>
#include <netdb.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <unistd.h>
#ifdef __FreeBSD__
#include <stdlib.h>
#else
#include <malloc.h>
#endif
#include <memory.h>
#include "chario.h"
#include "macros.h"
#include "services.h"
#include "classes.h"
#include "groups.h"

u_int16_t nextid = 1;


static int comphash(PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
		    PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
		    PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
		    PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
		    PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE]);
static int locate_class(PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
			PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
			PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
			PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE],
			PortListEntry [LIST_SIZE], PortListEntry [LIST_SIZE]);
static int compare_entry(PortListEntry *, PortListEntry *);
static int coalesce(PortListEntry [], int);
static void filltable(PortListEntry [LIST_SIZE], struct port_range *);


struct classtable {
     struct classtable *next;
     int id;
     int hash;
     PortListEntry t_dst_in[LIST_SIZE];
     PortListEntry t_dst_out[LIST_SIZE];
     PortListEntry t_src_in[LIST_SIZE];
     PortListEntry t_src_out[LIST_SIZE];
     PortListEntry u_dst_in[LIST_SIZE];
     PortListEntry u_dst_out[LIST_SIZE];
     PortListEntry u_src_in[LIST_SIZE];
     PortListEntry u_src_out[LIST_SIZE];
     PortListEntry i_typ_in[LIST_SIZE];
     PortListEntry i_typ_out[LIST_SIZE];
};

struct classtable *classlist = NULL;
struct classtable *classlist_tail = NULL;

static int maxclasslen = 0;
static int minclasslen = LIST_SIZE + 1;
static int totclasslen = 0;
static int totclasses = 0;

void
writeclassestable(int fd)
{
	PortListEntry class [10] [LIST_SIZE];
	struct classtable *rove;
	u_int16_t i,j,k,t[2];

	i = htons(nextid);
	write(fd, &i, sizeof(i));

	printf("%d class%s (min/avg/max = %d/%d/%d)\n", nextid-1,
		(nextid-1 == 1) ? "" : "es",
		(minclasslen == LIST_SIZE + 1) ? 0 : minclasslen,
		totclasses ? (totclasslen+(totclasses/2))/totclasses : 0,
		maxclasslen);


	add_default_class();

	for (rove=classlist,i=0; rove; rove=rove->next,i++) {
		if (rove->id != i) {
			fprintf(stderr, "%s, line %d: internal error, "
				"classlist is in unexpted order (%d,%d)\n",
				__FILE__, __LINE__, rove->id, i);
			exit(1);
		}
		memcpy(class[0], rove->t_dst_in, sizeof(class[0]));
		memcpy(class[1], rove->t_dst_out, sizeof(class[1]));
		memcpy(class[2], rove->t_src_in, sizeof(class[2]));
		memcpy(class[3], rove->t_src_out, sizeof(class[3]));
		memcpy(class[4], rove->u_dst_in, sizeof(class[4]));
		memcpy(class[5], rove->u_dst_out, sizeof(class[5]));
		memcpy(class[6], rove->u_src_in, sizeof(class[6]));
		memcpy(class[7], rove->u_src_out, sizeof(class[7]));
		memcpy(class[8], rove->i_typ_in, sizeof(class[8]));
		memcpy(class[9], rove->i_typ_out, sizeof(class[9]));
		for (j=0; j<10; j++) {
			for (k=0; k<LIST_SIZE; k++) {
				t[0] = htons(class[j][k].begin);
				t[1] = htons(class[j][k].end);
				write(fd, t, sizeof(t));
				if (k == class[j][0].begin)
					break;
			}
		}
	}
}
     
static int
comphash(PortListEntry t_dst_in[LIST_SIZE],
	 PortListEntry t_dst_out[LIST_SIZE],
	 PortListEntry t_src_in[LIST_SIZE],
	 PortListEntry t_src_out[LIST_SIZE],
	 PortListEntry u_dst_in[LIST_SIZE],
	 PortListEntry u_dst_out[LIST_SIZE],
	 PortListEntry u_src_in[LIST_SIZE],
	 PortListEntry u_src_out[LIST_SIZE],
	 PortListEntry i_typ_in[LIST_SIZE],
	 PortListEntry i_typ_out[LIST_SIZE])
{
     int i;
     int hash;

     hash = 0;

     for(i=0; i<LIST_SIZE; i++){
	  hash += t_dst_in[i].begin + t_dst_in[i].end;
	  hash &= 0x00FFFFFF;
	  hash += t_dst_out[i].begin + t_dst_out [i].end;
	  hash &= 0x00FFFFFF;
	  hash += t_src_in[i].begin + t_src_in[i].end;
	  hash &= 0x00FFFFFF;
	  hash += t_src_out[i].begin + t_src_out [i].end;
	  hash &= 0x00FFFFFF;
	  hash += u_dst_in[i].begin + u_dst_in[i].end;
	  hash &= 0x00FFFFFF;
	  hash += u_dst_out[i].begin + u_dst_out [i].end;
	  hash &= 0x00FFFFFF;
	  hash += u_src_in[i].begin + u_src_in[i].end;
	  hash &= 0x00FFFFFF;
	  hash += u_src_out[i].begin + u_src_out [i].end;
	  hash &= 0x00FFFFFF;
	  hash += i_typ_in[i].begin + i_typ_in[i].end;
	  hash &= 0x00FFFFFF;
	  hash += i_typ_out[i].begin + i_typ_out[i].end;
	  hash &= 0x00FFFFFF;
     }
     return hash;
}

static int
locate_class(PortListEntry t_dst_in[LIST_SIZE],
	     PortListEntry t_dst_out[LIST_SIZE],
	     PortListEntry t_src_in[LIST_SIZE],
	     PortListEntry t_src_out[LIST_SIZE],
	     PortListEntry u_dst_in[LIST_SIZE],
	     PortListEntry u_dst_out[LIST_SIZE],
	     PortListEntry u_src_in[LIST_SIZE],
	     PortListEntry u_src_out[LIST_SIZE],
	     PortListEntry i_typ_in[LIST_SIZE],
	     PortListEntry i_typ_out[LIST_SIZE])
{
     struct classtable *rove;
     int match = 0;

     int hash = comphash(t_dst_in, t_dst_out, t_src_in, t_src_out,
			 u_dst_in, u_dst_out, u_src_in, u_src_out,
			 i_typ_in, i_typ_out);

     for (rove=classlist; rove; rove=rove->next)
	  if (hash == rove->hash) {
	       int i;
	       match = 1;
	       for (i=0; i<LIST_SIZE; i++) {
		    if(rove->t_dst_in[i].begin != t_dst_in[i].begin ||
			    rove->t_dst_in[i].end != t_dst_in[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->t_dst_out[i].begin != t_dst_out[i].begin ||
		       rove->t_dst_out[i].end != t_dst_out[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->t_src_in[i].begin != t_src_in[i].begin ||
		       rove->t_src_in[i].end != t_src_in[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->t_src_out[i].begin != t_src_out[i].begin ||
		       rove->t_src_out[i].end != t_src_out[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->u_dst_in[i].begin != u_dst_in[i].begin ||
		       rove->u_dst_in[i].end != u_dst_in[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->u_dst_out[i].begin != u_dst_out[i].begin ||
		       rove->u_dst_out[i].end != u_dst_out[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->u_src_in[i].begin != u_src_in[i].begin ||
		       rove->u_src_in[i].end != u_src_in[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->u_src_out[i].begin != u_src_out[i].begin ||
		       rove->u_src_out[i].end != u_src_out[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->i_typ_in[i].begin != i_typ_in[i].begin ||
		       rove->i_typ_in[i].end != i_typ_in[i].end){
			 match = 0;
			 break;
		    }
		    if(rove->i_typ_out[i].begin != i_typ_out[i].begin ||
		       rove->i_typ_out[i].end != i_typ_out[i].end){
			 match = 0;
			 break;
		    }
	       }
	       if(match){
		    match = rove->id;
		    break;
	       }
	  }
     if(!rove){
	  struct classtable *new;
	  new = (struct classtable *)malloc(sizeof(struct classtable));
	  if(!new){
	       fprintf(stderr, "%s, line %d: malloc failed\n",
		       __FILE__, __LINE__);
	       exit(1);
	  }
	  new->id = nextid++;
	  new->hash = hash;
	  memcpy(new->t_dst_in,  t_dst_in,  sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->t_dst_out, t_dst_out, sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->t_src_in,  t_src_in,  sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->t_src_out, t_src_out, sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->u_dst_in,  u_dst_in,  sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->u_dst_out, u_dst_out, sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->u_src_in,  u_src_in,  sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->u_src_out, u_src_out, sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->i_typ_in,  i_typ_in,  sizeof(PortListEntry) * LIST_SIZE);
	  memcpy(new->i_typ_out, i_typ_out, sizeof(PortListEntry) * LIST_SIZE);
	  new->next = NULL;
	  if (classlist_tail == NULL)
		classlist = new;
	  else
		classlist_tail->next = new;
	  classlist_tail = new;

	  match = new->id;
     }
     return match;
}


void
add_default_class(void)
{
	struct classtable *new;

	if ((classlist != NULL) && (classlist->id == 0))
		return;

	new = (struct classtable *)calloc(1, sizeof(struct classtable));
	if(!new){
		fprintf(stderr, "%s, line %d: malloc failed\n",
				__FILE__, __LINE__);
		exit(1);
	}

	if (locate_group("default")) {
		push_group("default");
		gen_class(new->t_dst_in, new->t_dst_out, new->t_src_in,
			  new->t_src_out, new->u_dst_in, new->u_dst_out,
			  new->u_src_in, new->u_src_out, new->i_typ_in,
			  new->i_typ_out);
		destroy_all_services();
	}

	new->id = 0;
	new->hash = comphash(new->t_dst_in, new->t_dst_out, new->t_src_in,
			     new->t_src_out, new->u_dst_in, new->u_dst_out,
			     new->u_src_in, new->u_src_out, new->i_typ_in,
			     new->i_typ_out);
	new->next = classlist;
	classlist = new;
	if (classlist_tail == NULL)
		classlist_tail = classlist;
}

static int
compare_entry(PortListEntry *x, PortListEntry *y)
{
     if(x->begin < y->begin)
	  return -1;
     else
	  return 1;
}

#if 0
static void
zeroempty(PortListEntry buf[LIST_SIZE])
{
     int i;
     for(i=LIST_SIZE-1;i>=0;i--)
	  if(buf[i].begin == 65535 && buf[i].end == 65535)
	       buf[i].begin = buf[i].end = 0;
	  else
	       break;
}
#endif

static int
coalesce(PortListEntry buf[], int size)
{
     int ndx, i;
     size--;

     for(ndx=0;;){
	  if(ndx == size)
	       break;
	  if(buf[ndx].end+1 == buf[ndx+1].begin){
	       buf[ndx].end = buf[ndx+1].end;
	       for(i=ndx+1;i<size;i++)
		    buf[i] = buf[i+1];
	       size--;
	  }
	  else
	       ndx++;
     }
     return ndx+1;
}

static void 
filltable(PortListEntry buf[LIST_SIZE], struct port_range *list)
{
     struct port_range *rove;
     PortListEntry *build;
     int buildsize;
     int ndx;

     buildsize = 16;
     build = (PortListEntry *)malloc(sizeof(PortListEntry)*buildsize);
     if(!build){
	  fprintf(stderr, "%s, line %d: malloc failed\n", __FILE__, __LINE__);
	  exit(1);
     }
     for(ndx=0,rove=list;  rove;  rove=rove->pr_next){
	  if(rove->pr_begin != 65535 || rove->pr_end != 65535){
	       if(ndx == buildsize){
		    buildsize *= 2;
		    build = (PortListEntry *)
			 realloc(build,sizeof(PortListEntry)*buildsize);
		    if(!build){
			 fprintf(stderr, "%s, line %d: realloc failed\n",
				 __FILE__, __LINE__);
			 exit(1);
		    }
			 
	       }
	       build[ndx].begin = rove->pr_begin;
	       build[ndx].end = rove->pr_end;
	       ndx++;
	  }
     }

     memset(buf, 0, LIST_SIZE*sizeof(PortListEntry));
     if (ndx) {
	  qsort(build, ndx, sizeof(PortListEntry), (void *) compare_entry);
	  ndx = coalesce(build, ndx);

	  /*
	   * This is LIST_SIZE minus 1 because the first entry in the class
	   * table is reserved for internal use.
	   */
	  if(ndx > LIST_SIZE-1) {
	       fprintf(stderr, "\"%s\", line %d: Filter class too large " \
	       		"(%d entries, max is %d).\n",
			getfilename(), getlinenum(), ndx, LIST_SIZE-1);
	       exit(1);
	  }
	  memcpy(&(buf[1]), build, ndx * sizeof(PortListEntry));

	  /* save the number of entries in the table in the first slot */
	  buf[0].begin = ndx;
     }

     if(ndx < minclasslen)
	  minclasslen = ndx;
     if(ndx > maxclasslen)
	  maxclasslen = ndx;
     totclasslen += ndx;
     totclasses++;
}

void
gen_class(PortListEntry t_dst_in[LIST_SIZE],
	  PortListEntry t_dst_out[LIST_SIZE],
	  PortListEntry t_src_in[LIST_SIZE],
	  PortListEntry t_src_out[LIST_SIZE],
	  PortListEntry u_dst_in[LIST_SIZE],
	  PortListEntry u_dst_out[LIST_SIZE],
	  PortListEntry u_src_in[LIST_SIZE],
	  PortListEntry u_src_out[LIST_SIZE],
	  PortListEntry i_typ_in[LIST_SIZE],
	  PortListEntry i_typ_out[LIST_SIZE])
{
     struct filt_protocols *fp_rove;

     /* Clear the tables */
     memset(t_dst_in,  0, sizeof(PortListEntry) * LIST_SIZE);
     memset(t_dst_out, 0, sizeof(PortListEntry) * LIST_SIZE);
     memset(t_src_in,  0, sizeof(PortListEntry) * LIST_SIZE);
     memset(t_src_out, 0, sizeof(PortListEntry) * LIST_SIZE);
     memset(u_dst_in,  0, sizeof(PortListEntry) * LIST_SIZE);
     memset(u_dst_out, 0, sizeof(PortListEntry) * LIST_SIZE);
     memset(u_src_in,  0, sizeof(PortListEntry) * LIST_SIZE);
     memset(u_src_out, 0, sizeof(PortListEntry) * LIST_SIZE);
     memset(i_typ_in,  0, sizeof(PortListEntry) * LIST_SIZE);
     memset(i_typ_out, 0, sizeof(PortListEntry) * LIST_SIZE);

     /* */
     remove_deny(FP_IN,  FP_SRC);
     remove_deny(FP_IN,  FP_DST);
     remove_deny(FP_OUT, FP_SRC);
     remove_deny(FP_OUT, FP_DST);

     for (fp_rove=fp_head; fp_rove; fp_rove=fp_rove->fp_next)
	  switch (fp_rove->fp_id) {
	  case IPPROTO_TCP:
	       filltable(t_dst_in,  fp_rove->fp_pr[FP_ACC][FP_IN][FP_DST]);
	       filltable(t_dst_out, fp_rove->fp_pr[FP_ACC][FP_OUT][FP_DST]);
	       filltable(t_src_in,  fp_rove->fp_pr[FP_ACC][FP_IN][FP_SRC]);
	       filltable(t_src_out, fp_rove->fp_pr[FP_ACC][FP_OUT][FP_SRC]);
	       break;
	  case IPPROTO_UDP:
	       filltable(u_dst_in,  fp_rove->fp_pr[FP_ACC][FP_IN][FP_DST]);
	       filltable(u_dst_out, fp_rove->fp_pr[FP_ACC][FP_OUT][FP_DST]);
	       filltable(u_src_in,  fp_rove->fp_pr[FP_ACC][FP_IN][FP_SRC]);
	       filltable(u_src_out, fp_rove->fp_pr[FP_ACC][FP_OUT][FP_SRC]);
	       break;
	  case IPPROTO_ICMP:
	       filltable(i_typ_in,  fp_rove->fp_pr[FP_ACC][FP_IN][FP_DST]);
	       filltable(i_typ_out, fp_rove->fp_pr[FP_ACC][FP_OUT][FP_DST]);
	       break;
	  default:
	       break;
	  }
}

int
make_class(void)
{
     PortListEntry t_dst_in[LIST_SIZE];
     PortListEntry t_dst_out[LIST_SIZE];
     PortListEntry t_src_in[LIST_SIZE];
     PortListEntry t_src_out[LIST_SIZE];
     PortListEntry u_dst_in[LIST_SIZE];
     PortListEntry u_dst_out[LIST_SIZE];
     PortListEntry u_src_in[LIST_SIZE];
     PortListEntry u_src_out[LIST_SIZE];
     PortListEntry i_typ_in[LIST_SIZE];
     PortListEntry i_typ_out[LIST_SIZE];
     int i;

     for(i=0;i<LIST_SIZE;i++){
	  t_dst_in[i].begin  = t_dst_in[i].end = 0;
	  t_dst_out[i].begin = t_dst_out[i].end = 0;
	  t_src_in[i].begin  = t_src_in[i].end = 0;
	  t_src_out[i].begin = t_src_out[i].end = 0;
	  u_dst_in[i].begin  = u_dst_in[i].end = 0;
	  u_dst_out[i].begin = u_dst_out[i].end = 0;
	  u_src_in[i].begin  = u_src_in[i].end = 0;
	  u_src_out[i].begin = u_src_out[i].end = 0;
	  i_typ_in[i].begin  = i_typ_in[i].end = 0;
	  i_typ_out[i].begin = i_typ_out[i].end = 0;
     }

     gen_class(t_dst_in, t_dst_out, t_src_in, t_src_out,
	       u_dst_in, u_dst_out, u_src_in, u_src_out,
	       i_typ_in, i_typ_out);

     return locate_class(t_dst_in, t_dst_out, t_src_in, t_src_out,
			 u_dst_in, u_dst_out, u_src_in, u_src_out,
			 i_typ_in, i_typ_out);
}
