/*
 * Copyright (c) 1993,1994,1995,1997,1998
 *      Texas A&M University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Texas A&M University
 *      and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Developers:
 *     Russell Neeper, David K. Hess, Douglas Lee Schales, David R. Safford
 */

#ifndef __HOSTS_H__
#define __HOSTS_H__

#include "dbfc.h"
#include "classes.h"

struct hostrange {
	u_int32_t begin;
	u_int32_t end;
};

struct host_def {
     char *hfile;
     char *nfile;
     unsigned int hline;
     unsigned int nline;
};

struct hosttable {
     struct hosttable *next;
     u_int32_t addr;
     u_int8_t bits;
     u_int16_t class;
     struct host_def *def;
};

struct accept {
     u_int32_t ipaddr;
     u_int16_t bits;
     u_int16_t flag;
};

struct reject {
     u_int32_t ipaddr;
     u_int16_t bits;
     u_int16_t flag;
};

struct override {
     u_int32_t ipaddr;
     u_int32_t netmask;
     PortListEntry ports[LIST_SIZE];
};

typedef enum { INVERT=0, NORMAL=1 } norminv;

void init_hosts(void);
void writetables(char *);
struct host_def *check_host(u_int32_t);
void push_ip(u_int32_t);
void do_host(void);
void do_network(struct hostrange, unsigned int, int);
void do_tcpreset(void);
void add_accept(struct hostrange, unsigned int, int, norminv);
void add_reject(struct hostrange, unsigned int, int, norminv);
void add_override(unsigned int, unsigned int, int);

#endif
