/*
 * Copyright (c) 1993,1994,1995,1997,1998
 *      Texas A&M University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Texas A&M University
 *      and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Developers:
 *     Russell Neeper, David K. Hess, Douglas Lee Schales, David R. Safford
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "drawbridge.h"
#include "dbmgr.h"
#include "tokens.h"
#include "misc.h"

#define __CLEAR__
#include "text.h"

/*------------------------------------------------------------*/

void
usageClear(int verbose)
{
        fprintf(theOutput, "%s%s",
                (verbose) ? clearText[0] : clearText[0]+1,
                (verbose) ? clearText[1] : "");
}

/*------------------------------------------------------------*/

void
handleClear()
{
	int result;
	char *currTok;
	manageTablesReq mreq;

	currTok = strtok(NULL, DELIMITERS);
	result = findtoken(currTok);

	if (currTok && (*currTok == '?')) {
		usageClear(VERBOSE);
		return;
	}

	switch (result) {

	case T_FILTERS:
		/* Clear all tables */
		mreq.command = CLEAR_ALL;
		if (ioctl(theSocket, DIOCMNETWORK, &mreq)) {
			dfm_perror("DIOCMNETWORK");
			break;
		}
		break;

	case T_STATS:
		/* Send a clear stats request */
		if (ioctl(theSocket, DIOCCSTATS))
			dfm_perror("DIOCCSTATS");
		break;

	case T_BRIDGE:
		/* Send a clear bridge table request */
		if (ioctl(theSocket, DIOCCBRIDGE))
			dfm_perror("DIOCCBRIDGE");
		break;

	default:
		usageClear(TERSE);
	}
}

/*------------------------------------------------------------*/
