/*
 * Copyright (c) 1993,1994,1995,1997,1998
 *      Texas A&M University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Texas A&M University
 *      and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Developers:
 *     Russell Neeper, David K. Hess, Douglas Lee Schales, David R. Safford
 */

/*------------------------------------------------------------*/

#ifdef __CONTROL__

static char *initText[] = {
"\n"
"   INIT <inside device> <outside device> [ <mirror device> ]\n",
/*--------*/
"\n"
"   The 'init' command has two required and one optional parameters.  The\n"
"   required parameters are the device name of the inside interface and the\n"
"   device name of the outside interface.  The optional parameter is the \n"
"   name of the mirror device.  The mirror device may be used to mirror \n"
"   packets to a third interface for monitoring when Drawbridge is used \n"
"   in a full duplex environment.  See the 'set' command to configure \n"
"   mirroring.\n"
"\n"
};

static char *startText[] = {
"\n"
"   START\n",
/*--------*/
"\n"
"   The 'start' command will enable the drawbridge filter engine within the\n"
"   FreeBSD kernel.  No packets will pass between the two interfaces until\n"
"   drawbridge has been started.\n"
"\n"
};

static char *stopText[] = {
"\n"
"   STOP\n",
/*--------*/
"\n"
"   The 'stop' command will disable the drawbridge filter engine within the\n"
"   FreeBSD kernel.  No packets will pass between the two interfaces while\n"
"   drawbridge is stopped.\n"
"\n"
};

#endif

/*------------------------------------------------------------*/

#ifdef __SET__

static char *setText[] = {
"\n"
"   SET (LogFacility | LogMask | ListenInterface | MirrorMode) <value>\n",
/*--------*/
"   SET (MULticast | NonIP | OtherIP | SuspectOffset |\n"
"        FragmentedICMP | AttackICMP) (FORward | DIScard)\n",
/*--------*/
"\n"
"   The 'set' command is used to set global variables or flags.\n"
"   Type 'HELP SET <variable | flag>' for more information.\n"
"\n",
/*--------*/
"\n"
"   SET LogFacility <val>\n\n"
"   The 'LogFacility' variable controls which messages are generated by\n"
"   drawbridge to be passed to the unix syslog daemon for processing.\n"
"   The log facility that drawbridge uses may be set to any valid syslog\n"
"   facility.  Some examples are: 'user', 'daemon', 'local0'.\n"
"\n",
/*--------*/
"\n"
"   SET LogMask <val>\n\n"
"   The 'LogMask' variable controls which messages will be generated by the\n"
"   drawbridge filter engine.  The value of the mask is a hexadecimal or \n"
"   decimal number such as 0x123ABC or 1194684.  The default log mask is 0\n"
"   which disables all filter messages.  Each filter message may be enabled\n"
"   or disabled by setting or clearing the appropriate bit in the log mask.\n\n"
"   The following table of syslog messages and values may be used to\n"
"   generate a mask.  Simply add the decimal values (or bitwise OR the\n"
"   hex values) for the messages that you wish to receive and set LogMask\n"
"   to that value.\n\n"
"   Message                                        Hex Value  (Decimal)\n"
"   --------------------------------------------------------------------\n"
"   unknown event                                  0x00000001 (       1)\n"
"   cold start                                     0x00000002 (       2)\n"
"   incoming class D ...                           0x00000004 (       4)\n"
"   outgoing class D ...                           0x00000008 (       8)\n"
"   incoming port ...                              0x00000010 (      16)\n"
"   outgoing port ...                              0x00000020 (      32)\n"
"   incoming type ...                              0x00000040 (      64)\n"
"   outgoing type ...                              0x00000080 (     128)\n"
"   incoming via reject table ...                  0x00000100 (     256)\n"
"   outgoing via accept table ...                  0x00000200 (     512)\n"
"   outgoing via override table ...                0x00000400 (    1024)\n"
"   incoming header too short ...                  0x00000800 (    2048)\n"
"   outgoing header too short ...                  0x00001000 (    4096)\n"
"   incoming D-O-S attack ...                      0x00002000 (    8192)\n"
"   outgoing D-O-S attack ...                      0x00004000 (   16384)\n"
"   incoming IP ...                                0x00008000 (   32768)\n"
"   outgoing IP ...                                0x00010000 (   65536)\n"
"   incoming fragment with IP offset == 1 ...      0x00020000 (  131072)\n"
"   outgoing fragment with IP offset == 1 ...      0x00040000 (  262144)\n"
"   incoming fragment ...                          0x00080000 (  524288)\n"
"   outgoing fragment  ...                         0x00100000 ( 1048576)\n"
"   incoming MAC layer protocol                    0x00200000 ( 2097152)\n"
"   outgoing MAC layer protocol                    0x00400000 ( 4194304)\n"
"\n",
/*--------*/
"\n"
"   SET ListenInterface (InSide | OutSide | BOth | NOne)\n\n"
"   Sets the interface(s) which Drawbridge may use to communicate\n"
"   with other hosts.  If listening is not enabled for an interface,\n"
"   bridging will occur normally but the Drawbridge host will not be\n"
"   able to send or receive packets to or from that interface.  If\n"
"   listening is not enabled for either interface, the system will be\n"
"   completely isolated from the network.\n"
"\n",
/*--------*/
"\n"
"   SET MirrorMode (DISabled | InSide | OutSide | BRidged | FILtered |\n"
"                   FILtered_InSide | FILtered_OutSide | BRidged_FILtered |\n"
"                   BRidged_FILtered_InSide | BRidged_FILtered_OutSide)\n\n"
"   Determines which packets will be sent to the mirror interface.\n\n"
"   DISabled   - no packets\n"
"   InSide     - packets received or sent by the inside interface\n"
"   OutSide    - packets received or sent by the outside interface\n"
"   BRidged    - packets which are forwarded between interfaces\n"
"   FILtered   - packets which were discarded by any filter\n"
"   FILtered_InSide   - packets which were discarded by an inside filter\n"
"   FILtered_OutSide  - packets which were discarded by an outside filter\n"
"   BRidged_FILtered  - any forwarded or discarded packet\n"
"   BRidged_FILtered_InSide  - any forwarded or discarded by inside filter\n"
"   BRidged_FILtered_OutSide - any forwarded or discarded by outside filter\n"
"\n",
/*--------*/
"\n"
"   SET MULticast (FORward | DIScard)\n\n"
"   The 'Multicast' flag controls whether IP multicast packets are forwarded\n"
"   or discarded.\n"
"\n",
/*--------*/
"\n"
"   SET NonIP (FORward | DIScard)\n\n"
"   The 'NonIP' flag controls whether protocol layer traffic other than IP,\n"
"   ARP, or RARP is forwarded or discarded.\n"
"\n",
/*--------*/
"\n"
"   SET OtherIP (FORward | DIScard)\n\n"
"   The 'OtherIP' flag controls whether IP layer traffic other than TCP, UDP,\n"
"   or ICMP is forwarded or discarded.\n"
"\n",
/*--------*/
"\n"
"   SET SuspectOffset (FORward | DIScard)\n\n"
"   The 'SuspectOffset' flag controls whether TCP/IP fragments with an offset\n"
"   of 1 should be forwarded or discarded.  This is mainly used to protect\n"
"   internal machines that may have poorly written IP fragment reassembly\n"
"   routines.  The default setting is 'forward' since discarding these\n"
"   packets could, though unlikely, discard valid traffic.\n"
"\n",
/*--------*/
"\n"
"   SET FragmentedICMP (FORward | DIScard)\n\n"
"   The 'FragmentedICMP' flag controls whether fragmented ICMP packets\n"
"   should be forwarded or discarded.  This is mainly used to block denial\n"
"   of service attacks which use fragmented ICMP echo reply packets.  For\n"
"   example, the \"ping of death\" is a denial of service attack of this\n"
"   type.  The default setting is 'forward' since discarding these packets\n"
"   could, though unlikely, discard valid traffic.\n"
"\n",
/*--------*/
"\n"
"   SET AttackICMP (FORward | DIScard)\n\n"
"   The 'AttackICMP' flag controls whether ICMP packets containing the\n"
"   characteristics of a few known ICMP attacks are forwarded or discarded.\n"
"   The default setting is 'forward' since discarding these packets could,\n"
"   though unlikely, discard valid traffic.\n"
"\n"
};

#endif

/*------------------------------------------------------------*/

#ifdef __LOAD__

static char *loadText[] = {
"\n"
"   Load FILters <filename>\n",
/*--------*/
"\n"
"   The 'load' command is used to load the filters file that is generated\n"
"   by the filter compiler ('db_filters' by default).  The loaded filters\n"
"   will take effect immediately.\n"
"\n"
};

#endif

/*------------------------------------------------------------*/

#ifdef __CLEAR__

static char *clearText[] = {
"\n"
"   CLeaR (FILters | STats | BRidge)\n",
/*--------*/
"\n"
"   The 'clear' command can be used to clear the filter tables, the stats,\n"
"   or the bridge table.  If the filter tables are cleared, drawbridge will\n"
"   fall back to the default filter tables.  The changes take effect\n"
"   immediately.\n"
"\n"
};

#endif

/*------------------------------------------------------------*/

#ifdef __SHOW__

static char *showText[] = {
"\n"
"   SHow (Host <host> | CLass <class | host> [w] | TCPreset |\n"
"         ACCept | REJect | OVerride | FLags | STats | LogFacility |\n"
"         LogMask | BRidge | ListenInterface | MirrorMode)\n",
/*--------*/
"\n"
"  The 'show' command will display the specified filter table information,\n"
"  flags, variables, statistics, or the bridge table.\n"
"\n"
};

#endif

/*------------------------------------------------------------*/

#ifdef __MONITOR__

static char *monitorText[] = {
"\n"
"   MONitor [ INTerval <seconds> ]\n",
/*--------*/
"\n"
"   The 'monitor' command will display a full screen of packet statistics\n"
"   which is updated once per second by default.  The update interval may\n"
"   be specified from 1 to 60 seconds.  The space bar will toggle screens\n"
"   to display a breakdown of the number of filtered packets.  The 'q' key\n"
"   will quit back to the dbmgr prompt.\n"
"\n"
};

#endif

/*------------------------------------------------------------*/

#ifdef __MISC__

static char *errormsg[] = {
   "Parameter out of range",			/* ERROR_OUT_OF_RANGE */
   "Invalid class specified",			/* ERROR_UNKNOWN_CLASS */
   "Invalid network specified",			/* ERROR_INVALID_NETWORK */
   "Table is full",				/* ERROR_TABLE_FULL */
   "Network Not found",				/* ERROR_NET_NOT_FOUND */
   "Unable to allocate memory",			/* ERROR_NO_MEMORY */
   "Invalid operation",				/* ERROR_INVALID_OP */
   "Interface is down or not running",		/* ERROR_IF_DOWN */
   "Error getting hardware address",		/* ERROR_GET_HW */
   "Error getting IP address",			/* ERROR_GET_IP */
   "Invalid interface",				/* ERROR_INVALID_IF */
   "Can not use the same interface",		/* ERROR_SAME_IF */
   "Interfaces must be of the same media type",	/* ERROR_DIFF_IF_TYPE */
   "Unsupported media type",			/* ERROR_BAD_IF_TYPE */
   "Inside interface is down or not running",	/* ERROR_INSIDE_DOWN */
   "Outside interface is down or not running",	/* ERROR_OUTSIDE_DOWN */
   "Listen requires that only one interface have an IP address assigned",
						/* ERROR_TWO_IP */
   "Listen requires that one interface have an IP address assigned",
						/* ERROR_NO_IP */
   "Invalid listen interface specified",	/* ERROR_INVALID_MODE */
   "Drawbridge has not been initialized",	/* ERROR_NOT_INITILIZED */
   "Error - Drawbridge is currently running",	/* ERROR_RUNNING */
   "Error - Drawbridge is not running",		/* ERROR_NOT_RUNNING */
   "Mirror interface is down or not running",	/* ERROR_MIRROR_DOWN */
   "Mirror device has not been configured",	/* ERROR_MIRROR_UNDEFINED */
   "Invalid mirror mode specified"		/* ERROR_MIRROR_MODE */
};

#endif

/*------------------------------------------------------------*/
