#!/bin/sh

###### NOTE: All Drawbridge configuration is now done in /etc/rc.conf. ######
###### You should look at that file before making changes here.        ######

PATH=/usr/bin:/usr/sbin:/bin:/sbin:/usr/local/bin:/usr/local/sbin
export PATH

# Read in the global system configuration file.
if [ -f /etc/defaults/rc.conf ]; then
	. /etc/defaults/rc.conf
elif [ -f /etc/rc.conf ]; then
	. /etc/rc.conf
fi

# Check if drawbridge is enabled
case $db_enable in
[Yy][Ee][Ss])
	ena="y"
	;;
esac
[ "$ena" != "y" ] && exit 0

if [ "$1" = "stop" ]; then
        dbmgr stop
        exit 0
elif [ "$1" = "start" ]; then
	echo -n ' drawbridge'
else
	exit 1
fi

# Clear the boot log
[ ! "$db_boot_log" ] && db_boot_log=/dev/null
>$db_boot_log

# Make sure the required variables are defined
if [ ! "$db_inside_if" -o ! "$db_outside_if" -o ! "$db_filters_file" ]; then
	printf "\n\n   [1;31m%s[0m\n\n" \
		"!! Drawbridge is not configured - edit /etc/rc.conf !!" \
	| tee $db_boot_log
	exit 0
fi

# Optionally reconfigure the inside interface
if [ "$db_ifconfig_inside_if" ]; then
	ifconfig $db_inside_if $db_ifconfig_inside_if >> $db_boot_log
fi

# Optionally reconfigure the outside interface
if [ "$db_ifconfig_outside_if" ]; then
	ifconfig $db_outside_if $db_ifconfig_outside_if >> $db_boot_log
fi

# Optionally reconfigure the mirror interface
if [ "$db_ifconfig_mirror_if" ]; then
	ifconfig $db_mirror_if $db_ifconfig_mirror_if >> $db_boot_log
fi

# Make sure that all of the interfaces are up and running
for IF in $db_inside_if $db_outside_if $db_mirror_if; do
	IFSTATE=`ifconfig $IF`
	if [ $? -ne 0 ]; then
		echo "ERROR: interface '$IF' does not exist"
		exit 1
	fi
	RUNNING=`echo $IFSTATE | egrep -c "[^A-Z]RUNNING[^A-Z]"`
	UP=`echo $RUNNING | egrep -c "[^A-Z]UP[^A-Z]"`
	if [ 0$RUNNING -eq 0  -o  0$UP -eq 0 ]; then
		ifconfig $IF up >> $db_boot_log
		if [ $? -ne 0 ]; then
			echo "ERROR: Unable to start interface '$IF'"
			exit 1
		fi
	fi
done

# Wait for the above ifconfig's to finish - needed by (at least) the ti driver
sleep 1

# If specified, set the log facility
if [ "$db_log_facility" ]; then
	dbmgr set logFacility $db_log_facility >> $db_boot_log
fi

# If specified, set the log mask
if [ "$db_log_mask" ]; then
	dbmgr set logMask $LOG_MASK >> $db_boot_log
fi

# Initialize Drawbridge
dbmgr init $db_inside_if $db_outside_if $db_mirror_if >> $db_boot_log

# If specified, set the mirror mode
if [ "$db_mirror_mode" ]; then
	dbmgr set mirrormode $db_mirror_mode >> $db_boot_log
fi

# If specified, set the listen interface
if [ "$db_listen_if" ]; then
	dbmgr set listeninterface $db_listen_if >> $db_boot_log
fi

# Load the filter file
dbmgr load filters $db_filters_file >> $db_boot_log

# Start Drawbridge
dbmgr start >> $db_boot_log
