#!/bin/sh

MANAGER_UID=2000
MONITOR_UID=2001

[ ! _$2 = _"POST-INSTALL" ] && exit 0
MAJORVER=`uname -r | cut -b1`


# ========================================================================
# Modify /etc/rc.conf
# ========================================================================
if [ ! -e /etc/rc.conf.orig ]; then
     [ -e /etc/rc.conf ] && cp -p /etc/rc.conf /etc/rc.conf.orig
fi
cat << EOF >> /etc/rc.conf

# -- drawbridge generaged deltas -- #
inetd_enable="NO"
portmap_enable="NO"
router_enable="NO"
sendmail_enable="NO"
syslogd_flags="-s"
local_startup="/usr/local/drawbridge/etc/rc.d \$local_startup"

# -- drawbridge configuration settings -- #
db_enable="YES"

# Uncomment the following and set them to the device names for your system
# You can determine the device names by running "ifconfig -a".
#db_inside_if="fxp0"	#device name of the inside interface (required)
#db_outside_if="fxp1"	#device name of the outside interface (required

# Optional ifconfig settings for the inside and outside interfaces
#db_ifconfig_inside_if="media 100baseTX mediaopt full-duplex"
#db_ifconfig_outside_if="media 100baseTX mediaopt full-duplex"

# Optional mirror interface setings
#db_mirror_if="fxp2"			#device name of the mirror interface
#db_ifconfig_mirror_if="media 100baseTX"
#db_mirror_mode="Bridged_Filtered"	#See dbmgr documentation for details

# If you are planning to compile filters on this system, you should enable
# listening on the side that has your DNS server.  Remote ssh access
# also requires listening to be enabled for the correct interface.  If
# listening is disabled for an interface, then *no* packets from that
# interface will be passed up to the kernel - including ARP packets.
# Setting db_listen_if to "none" will completely isolate the Drawbridge
# host from the network.  Comment out db_listen_if if neither drawbridge
# interface has an IP address assigned.
db_listen_if="inside"	#"none", "inside", "outside", or "both" (optional)

db_log_facility="local0"	#syslog facility (optional)
db_log_mask="0"			#syslog log mask (optional)

db_filters_file="/usr/local/drawbridge/etc/db_filters"	#compiled filter file

EOF


# ========================================================================
# Modify /etc/rc
# ========================================================================
[ ! -e /etc/rc.orig ] && mv /etc/rc /etc/rc.orig
sed	-e "/^# [Ss]napshot any kernel/,/^# [Tt]his has changed with ELF/c\\
# Snapshot any kernel -c changes back to disk here\\
kget /boot/kernel.conf\\
" /etc/rc.orig > /etc/rc 


# ========================================================================
# Modify the file /etc/crontab
# ========================================================================
[ ! -e /etc/crontab.orig ] && mv /etc/crontab /etc/crontab.orig
sed	-e "/^# do daily.*maintenance$/i\\
# For Drawbridge, the sendmail daemon is not running.\\
# This will process outgoing mail once an hour in case any gets queued.\\
0	*	*	*	*	root	/usr/sbin/sendmail -q\\
#\\
# Since Drawbridge is not a multi-user system, it is not necessary, and\\
# may be disruptive, to perform the standard maintenance tasks so they\\
# have been commented out.\\
"	-e "s/\(^[05].*daily.*\)/#\1/" \
	-e "s/\(^30.*weekly.*\)/#\1/" \
	-e "s/\(^30.*monthly.*\)/#\1/" /etc/crontab.orig > /etc/crontab


# ========================================================================
# Modify the file /etc/syslog.conf
# ========================================================================
[ ! -e /etc/syslog.conf.orig ] && mv /etc/syslog.conf /etc/syslog.conf.orig
sed -e "s/notice;mail.crit	/notice;mail.crit;local0.none/" \
    -e "s/;news.err	/;local0.none	/" \
    -e "/^\*\.err.*root$/ i\\
local0.*					/var/log/drawbridge
" /etc/syslog.conf.orig > /etc/syslog.conf
touch /var/log/drawbridge
chown $MANAGER_UID.$MONITOR_UID /var/log/drawbridge
chmod 640 /var/log/drawbridge


# ========================================================================
# Modify the file /etc/newsyslog.conf
# ========================================================================
[ ! -e /etc/newsyslog.conf.orig ] && mv /etc/newsyslog.conf /etc/newsyslog.conf.orig
cp -f /etc/newsyslog.conf.orig /etc/newsyslog.conf
echo "# Added for Drawbridge" >> /etc/newsyslog.conf
echo "/var/log/drawbridge	manager.monitor 640  7     *    24    Z" >> \
	/etc/newsyslog.conf


# ========================================================================
# Modify the file /etc/host.conf if necessary
# ========================================================================
if [ `/usr/bin/grep -c "First try the /etc/hosts file" /etc/host.conf` = 0 ]; then
  [ ! -e /etc/host.conf.orig ] && mv /etc/host.conf /etc/host.conf.orig
  echo "# Default is to use /etc/hosts file first" > /etc/host.conf
  echo "hosts" >> /etc/host.conf
  echo "# If that doesn't work, then try the the nameserver" >> /etc/host.conf
  echo "bind" >> /etc/host.conf
fi


# ========================================================================
# Add the drawbridge device to /dev/MAKEDEV.local
# ========================================================================
[ ! -e /dev/MAKEDEV.local.orig ] && mv /dev/MAKEDEV.local /dev/MAKEDEV.local.orig
sed -e "s|^\(PATH=.*\)|\1:/usr/sbin:\$PATH|" \
    -e "/^case \$i in$/a\\
\\
drawbridge)\\
\	mknod drawbridge c 20 0\\
\	chown $MANAGER_UID.$MONITOR_UID drawbridge\\
\	chmod 640 drawbridge\\
\	;;\\
" /dev/MAKEDEV.local.orig > /dev/MAKEDEV.local
chmod 555 /dev/MAKEDEV.local


# ========================================================================
# Fixup the root, manager, and monitor Home Directories
# ========================================================================

[ ! -e /root/.cshrc.orig ] && mv /root/.cshrc /root/.cshrc.orig
cp -f dot.cshrc /root/.cshrc
cp -f dot.cshrc /home/manager/.cshrc
cp -f dot.cshrc /home/monitor/.cshrc
sed -e "/^# remove.*games.*/d" \
    -e "s/^\(PATH.*\)games:\(.*\)\/usr\/X11R6\/bin:\(.*\)/\1local\/sbin:\2\3/" \
    /home/manager/.profile > /home/manager/.p
mv /home/manager/.p /home/manager/.profile
sed -e "/^# remove.*games.*/d" \
    -e "s/^\(PATH.*\)games:\(.*\)\/usr\/X11R6\/bin:\(.*\)/\1local\/sbin:\2\3/" \
    /home/monitor/.profile > /home/monitor/.p
mv /home/monitor/.p /home/monitor/.profile
sed -e "s%^\(set path.*\)games\(.*\)/usr/X11R6/bin \(.*\)%\1local/sbin\2\3%" \
    /home/manager/.login > /home/manager/.l
mv /home/manager/.l /home/manager/.login
sed -e "s%^\(set path.*\)games\(.*\)/usr/X11R6/bin \(.*\)%\1local/sbin\2\3%" \
    /home/monitor/.login > /home/monitor/.l
mv /home/monitor/.l /home/monitor/.login
ln -s /usr/local/drawbridge /root/drawbridge
ln -f -s /usr/local/drawbridge /home/manager/drawbridge
ln -f -s /usr/local/drawbridge /home/monitor/drawbridge
> /home/manager/.rhosts
> /home/monitor/.rhosts
mkdir /home/manager/.ssh
mkdir /home/monitor/.ssh
> /home/manager/.ssh/authorized_keys
> /home/monitor/.ssh/authorized_keys
chown -R $MANAGER_UID.$MANAGER_UID /home/manager
chown -R $MONITOR_UID.$MONITOR_UID /home/monitor
chown -h $MANAGER_UID.$MANAGER_UID /home/manager/drawbridge
chown -h $MONITOR_UID.$MONITOR_UID /home/monitor/drawbridge


# ========================================================================
# Force kernel configuration for the first reboot
# ========================================================================

[ ! -e /boot/loader.conf.orig ] && mv /boot/loader.conf /boot/loader.conf.orig
cp -f /boot/loader.conf.orig /boot/loader.conf
echo "boot_userconfig=\"YES\"" >> /boot/loader.conf
cp -f firstboot.sh /usr/local/drawbridge/etc/rc.d
chmod 700 /usr/local/drawbridge/etc/rc.d/firstboot.sh


# ========================================================================
# Done
# ========================================================================

exit 0
